#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

################################################################################
# Created       : Martin Foster
# Modified      : 01-Jan-2007
################################################################################
#
# Contributors - Secondary access portal designed to be used by contributors to
#                access all of their specific features.
# Copyright (C) 2000-2007 Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3-3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

use CGI;							# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Number::Format qw(:subs);					# Number formatting library
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Filter;						# Filter handling
use Ethereal::Home;						# Homepage handling
use Ethereal::Login;						# Login functionality
use Ethereal::Menu;						# Consistent menu
use Ethereal::Mail;						# Mail handler
use Ethereal::Option;						# Option handler
use Ethereal::Param;						# Parameter control
use Ethereal::Template;						# Template handler

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 50;					# Maximum posts
$CGI::DISABLE_UPLOADS = 1;					# Disable uploads

################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $menu;							# Menu handle
my $param;							# Parameter hash
my $tmpl;							# Template handle

my $portal;							# Portal template
my $key;							# Selected key	

my %contrib;							# Contributors hash
my %gal;							# Gallery hash
my %srch;							# Built up macros list

################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect($cgi);

	# Set title
	$database->{'TITLE'} = $database->{'SYS'}{'TitContrib'};

	# Parameter handling
	$param    = new Ethereal::Param($database, $cgi);
	$param->GetParam();

	# Link with hash
	$database->GetHashContrib(\%contrib);
	$database->GetHashGallery(\%gal);

	# Create instances
	$tmpl  = new Ethereal::Template(\%contrib);
	$menu  = new Ethereal::Menu($database, $param);
	$login = new Ethereal::Login($database, $cgi, $param, $menu, \%contrib);

	# Enable custom header and footer
	$login->{'HEADER'} = $database->DocumentGetContribHeader();
	$login->{'FOOTER'} = $database->DocumentGetContribFooter();


	
	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# Collect information
		CollectGeneral($database, $param, \%contrib, \%srch);
		CollectHome($database, $param, $tmpl, \%contrib, \%srch);
		CollectGallery($database, $param, $tmpl, \%contrib, \%srch);	

		# Pull portal
		$portal = $database->DocumentGetContribPortal();


		# Search and replace
		foreach $key (sort(keys(%srch)))
		{
			# Regular expression
			$portal =~ s/$key/$srch{$key}/gs;
		}

		# Disable header rewrite
		$menu->{'HEADER'} = 'false';

		# Display completed portal
		print $menu->Rewrite($portal), "\n";
	}

################################################################################
# Sub-Routines
################################################################################

#####################
# Gallery Collection
#
# This subroutine will generates lists of all relevant galleries for a specific
# type.   Going through all three, this will generate the final macros needed
# for the template generation.

sub CollectGallery
{
	#####################
	# Data members

	my $database = shift;					# Database handler
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handle
	my $contrib  = shift;					# Contributor hash
	my $srch     = shift;					# Search and replace hash

	my $res;						# Results string
	my $statement;						# Database statement
	
	my $lview;						# Link for viewing
	my $iview;						# Inline parameters
	my $imanage;						# Inline parameters

	my $genre;						# Singular genre
	my $genres;						# Formatted genres
	my $priv;						# Privacy features
	my $type;						# Singular gallery type
	my $work;						# Amount of works

	my @genres;						# Genres for a gallery


	#####################
	# Program area

	# Viewing inline parameters
	$iview = $param->EmbedInline(
 	  USER         => $param->{'USER'}, 
 	  CRYPT        => $param->{'CRYPT'}
	);
					
	# Global define
	$srch->{'MGALVIEW'}   = $database->{'SYS'}{'LnkIntGallery'}    . $iview;
	$srch->{'MGALMANAGE'} = $database->{'SYS'}{'LnkIntSetGallery'} . $iview;
	

	# Search for relevant material
	$database->Pull(\$statement, "SELECT
		 g.GalleryID       AS \"GalleryID\",
		 g.GalleryName     AS \"GalleryName\",
		 g.GalleryPenName  AS \"GalleryPenName\",
		 g.GalleryRating   AS \"GalleryRating\",
		 g.GalleryPrivacy  AS \"GalleryPrivacy\",
		 g.GalleryType     AS \"GalleryType\",
		 g.GallerySub      AS \"GallerySub\",
		 getDate(g.GalleryCreated) AS \"GalleryCreated\",
		 getDate(g.GalleryUpdated) AS \"GalleryUpdated\"
		 FROM Gallery g, GalleryAccess a
		WHERE g.GalleryID=a.GalleryID
		  AND a.PuppeteerLogin=?
		ORDER BY g.GalleryName",

	  $param->{'USER'}
	);

	# Verification
	if ($res = $statement->fetchrow_hashref())
	{
		# Increment
		$srch->{"MGALCOUNT"} = 0;

		# Loop and adjust
		do
		{
			# Increment
			$srch->{"MGALCOUNT"}++;

				
			# Genres and links
			# Genre searching	
			$database->GetListEstablished(\@genres, $res->{'GalleryID'});

			# Format genre line
			$genres = join("/", @genres);

			# Generate link
			$lview = $res->{'GalleryType'} . '/'
			       . $res->{'GallerySub'}  . '/'
			       . $genres[0] . '/'
			       . $res->{'GalleryID'};


			# Adjust privacy
			$priv   = ($res->{'GalleryPrivacy'} eq 'yes')
			  ? $contrib->{'TagPrivTrue'}
			  : $contrib->{'TagPrivFalse'};

			# Count works
			($work) = $database->DataGet("SELECT COUNT(*)
				FROM Work
				WHERE GalleryID=?
				  AND WorkID > 1",

			 $res->{'GalleryID'}
			);
    	


			# Links
			# Management inline parameters
			$imanage = $param->EmbedInline(
	  		  USER         => $param->{'USER'}, 
	  		  CRYPT        => $param->{'CRYPT'},
			  GID          => $res->{'GalleryID'},
			  TASK         => 'galview'
			);
					
			# Append to list
			$srch->{"LSTGAL"} .= $tmpl->Pass('TmplGalleryList',
				MCREATE => $res->{'GalleryCreated'},
				MID     => $res->{'GalleryID'},
				MGNAME  => $res->{'GalleryName'},
				MPNAME  => $res->{'GalleryPenName'},
				MRATE   => $res->{'GalleryRating'},
				MUPDATE => $res->{'GalleryUpdated'},
				MGENRES => $genres,
				MPRIV   => $priv,
				MWRKCNT => $work,
				LVIEW   => $database->{'SYS'}{'LnkIntGallery'} . '/' . $lview . $iview,
				LMANAGE => $database->{'SYS'}{'LnkIntSetGallery'} . $imanage);

		} while ($res = $statement->fetchrow_hashref());
	}

	# Nothing to display
	else
	{
		# A blank template
		$srch->{"LSTGAL"} = $contrib->{'TmplMissing'};
	}
}

	
#####################
# General information
#
# Will gather general information specific to the user and seperate from the
# homepage or gallery components that are covered elsewhere.

sub CollectGeneral 
{
	#####################
	# Data members

	my $database = shift;					# Database handler
	my $param    = shift;					# Parameters
	my $contrib  = shift;					# Contributor hash
	my $srch     = shift;					# Search and replace hash


	#####################
	# Program area

	# Pull information
	($srch->{'MGENUSER'},
	 $srch->{'MGENNAME'},
	 $srch->{'MGENEMAIL'},
	 $srch->{'MGENVERIFY'},
	 $srch->{'MGENEXPIRE'}
 	) = $database->DataGet("SELECT
		 PuppeteerLogin     AS \"PuppeteerLogin\",
		 PuppeteerName      AS \"PuppeteerName\",
		 PuppeteerEmail     AS \"PuppeteerEmail\",
		 PuppeteerVerified  AS \"PuppeteerVerified\",
		 getDate(PuppeteerHomeSpan) AS \"PuppeteerHomeSpan\"
		FROM Puppeteer
       		WHERE PuppeteerLogin=?", $param->{'USER'});

	# Verified user formatting
	$srch->{'MGENVERIFY'} = ($srch->{'MGENVERIFY'} eq 'yes')
	  ? $contrib->{'TagVerTrue'}
	  : $contrib->{'TagVerFalse'};

	# General inline parameters
	$srch->{'LGENPARAM'} = $param->EmbedInline(
 	  USER         => $param->{'USER'}, 
 	  CRYPT        => $param->{'CRYPT'}
	);
}


#####################
# Homepage Collect 
#
# Homepage collection is a sub-routine that is designed to populate the hash
# with all of the necessary information in order to send off it's results to
# the template handler.

sub CollectHome
{
	#####################
	# Data members

	my $database = shift;					# Database handler
	my $param    = shift;					# Parameters
	my $tmpl     = shift;					# Template handler
	my $contrib  = shift;					# Contributor hash
	my $srch     = shift;					# Search and replace hash

	my $hp;							# Homepage handle
	my $inline;						# Inline management
	my $file;						# Singular file
	my $remain;						# Remaining space

	my @files;						# Pulled filenames
	my %home;						# Homepage hash


	#####################
	# Program area

	# Create instances
	$hp = new Ethereal::Home($database, $database->{'SYS'}{'SetHomeRoot'});
	
	# Inline parameters
	$inline = $param->EmbedInline(
	  USER  => $param->{'USER'}, 
	  CRYPT => $param->{'CRYPT'}
 	 );

	
	# Find and home directory
	($srch->{'MHPHOME'}) = $database->DataGetPuppeteerHome($param->{'USER'});

	# Homepage management link
	$srch->{'LHPMANAGE'} = $database->{'SYS'}{'LnkIntSetHome'} . $inline;

	# File limits
	$srch->{'MHPLIMIT'}  = format_bytes(($contrib->{'SetSize'} * 1048576), 2); 


	# Check for existence
	# Defined
	if (defined($srch->{'MHPHOME'}))
	{
		# Base information
		# Used space and count
		($srch->{'MHPSIZE'},
		 $srch->{'MHPCOUNT'}) = $hp->FileListing($srch->{'MHPHOME'}, \@files, $hp);

	 	# Remaining size calculation
	 	$remain = ($contrib->{'SetSize'} * 1048576) - $srch->{'MHPSIZE'};
		$remain = ($remain < 0) ? 0 : $remain;
		
	 	# Format accordingly
		$srch->{'MHPSIZE'}  = format_bytes($srch->{'MHPSIZE'}, 2);
		$srch->{'MHPAVAIL'} = format_bytes($remain, 2);
		$srch->{'MHPCOUNT'} = format_number($srch->{'MHPCOUNT'},0,0);
	 
	 	# Generate list
		# Initial assignment
		$srch->{'LSTHPFILES'} = '';
		
		# Loop and generate
		foreach $file (sort(@files))
		{
			# Append new works
			$srch->{'LSTHPFILES'} = $srch->{'LSTHPFILES'}
		       	. $tmpl->Pass('TmplFileList',
			       	MFNAME=> $file,
				MFLINK=> $database->{'SYS'}{'SetHomeLink'} . $srch->{'MHPHOME'} . '/' . $file);
		}
	}

	# Undefined
	else
	{
		# Define to base values
		$srch->{'MHPAVAIL'}   = $contrib->{'TextNA'};
		$srch->{'MHPCOUNT'}   = $contrib->{'TextNA'};
		$srch->{'MHPHOME'}    = $contrib->{'TextHome'};
		$srch->{'MHPSIZE'}    = $contrib->{'TextNA'};

		# List cheat
		$srch->{'LSTHPFILES'} = $contrib->{'TagNoFiles'};
	}

	# Return true
	return 1;
}
