#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

#################################################################################
# Created       : Martin Foster
# Modified      : 27-Oct-2006
#################################################################################
#
# Transfer - Generate SQL output in order to transfer from one site to another
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3-3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler


#################################################################################
# Data Members
#################################################################################
my $database;							# Database handle
my $old;							# Site name to transfer
my $new;							# New site transfer

#################################################################################
# Program Area
#################################################################################

	# Make sure schema is specified
	if ($old = shift(@ARGV))
	{
		# Additional schema specification
		if ($new = shift(@ARGV))
		{
			# Initial handles
			$database = new Ethereal::Database();

			# Connect and Fetch
			$database->Connect();
	
			# Set search paths
			$database->Write("SET SEARCH_PATH TO $old,public");

			# Generate file
			Generate($database, $new);

			# Inform
			print "Completed generation of SQL file for $new from $old\n\n";
		}
	}


#################################################################################
# Sub-Routines
#################################################################################


#####################
# Data Format
#
# Will make sure to return things properly for use in the display.   Otherwise
# things may not work as expected.

sub Format
{
	#####################
	# Data members

	my $data    = shift;					# Piece to check 


	#####################
	# Program area

	# Defined
	if (defined($data))
	{
		# Format cleanly
		$data =~ s/(\'|\"|\`)/\\$1/gs;
		$data =~ s/\n/\\n/gs;
		$data =~ s/\r/\\r/gs;
		$data =~ s/\\/\\/gs;

		# Return
		return "\'$data\'";
	}

	# Nulls
	else
	{
		# Null value
		return 'NULL';
	}
}

#####################
# Short Name
#
# Generates short names required for new versions of Ethereal Realms.

sub Short
{
	#####################
	# Data members

	my $full    = shift;					# Full name
	my $short   = shift;					# Short name


	#####################
	# Program area

	# Generate short name
	$short  =  lc($full);
	$short  =~ s/-/_/gs;
	$short  =~ s/\s/_/gs;
	$short  =~ s/\W//gs;

	# Return value
	return "\'$short\'";
}



#####################
# Generate
#
# Grabs from the old and hands off the the new.  Nothing more difficult then that
# and should be faily easy to adapt to older scripts.

sub Generate
{
	#####################
	# Data members
	my $data      = shift;					# Database handle
	my $default   = shift;					# New domain name

	my $file;						# Filename
	my $res;						# Result handle
	my $statement;						# Query

	#####################
	# Program area

	# Create filehandle
	open(FILE, ">./xfer.sql")
	 or die("Unable to open ./xfer.sql: $!");
	
	#
	#
	# Font
	$data->Pull(\$statement, "SELECT
		  FontName         AS \"FontName\",
		  FontType         AS \"FontType\",
		  FontCode         AS \"FontCode\"
		FROM Font
		WHERE FontType <> 'size'
		ORDER BY FontType, FontName");

	# Title
	print FILE "\n\n--\n-- FONT\n--\n";

	# Cleanup
	print FILE "DELETE FROM $default.Font WHERE FontType IN ('color','size');\n";

	# Loop and display
	while ($res = $statement->fetchrow_hashref())
	{
		# Display
		printf FILE "INSERT INTO $default.Font (FontName,FontType,FontCode) VALUES(%s,%s,%s);\n",
			Format($res->{'FontName'}),
			Format($res->{'FontType'}),
			Format($res->{'FontCode'});
	}

	# End
	$statement->finish();


	#
	#
	# Gummies
	$data->Pull(\$statement, "SELECT
		  GummyName        AS \"GummyName\",
		  GummyClass       AS \"GummyClass\",
		  GummyPath        AS \"GummyPath\",
		  GummyIdent       AS \"GummyIdent\",
		  GummyMacro       AS \"GummyMacro\"
		FROM Gummy
		WHERE RealmName=?
		  AND GummyIdent IS NULL
		ORDER BY GummyCLass, GummyName", 'system');

	# Title
	print FILE "\n\n--\n-- GUMMY\n--\n";

	# Cleanup
	print FILE "DELETE FROM $default.Gummy WHERE RealmName='system';\n";

	# Loop and display
	while ($res = $statement->fetchrow_hashref())
	{
		# Display
		printf FILE "INSERT INTO $default.Gummy (GummyName,GummyClass,GummyPath,GummyIdent,GummyMacro) VALUES(%s,%s,%s,%s,%s);\n",
			Format($res->{'GummyName'}),
			Format($res->{'GummyClass'}),
			Format($res->{'GummyPath'}),
			Format($res->{'GummyIdent'}),
			Format($res->{'GummyMacro'});
	}

	# End
	$statement->finish();


	#
	#
	# Users
	$data->Pull(\$statement, "SELECT
			PuppeteerLogin      AS \"PuppeteerLogin\",
			PuppeteerPassword   AS \"PuppeteerPassword\",
			PuppeteerName       AS \"PupeteerName\",
			PuppeteerEMail      AS \"PuppeteerEmail\",
			PuppeteerVerified   AS \"PuppeteerVerified\",
			PuppeteerAdminPrivs AS \"PuppeteerAdminPrivs\",
			PuppeteerSuperPrivs AS \"PuppeteerSuperPrivs\",
			PuppeteerModPrivs   AS \"PuppeteerModPrivs\",
			PuppeteerHome       AS \"PuppeteerHome\",
			PuppeteerHomeSpan   AS \"PuppeteerHomeSpan\"
		FROM Puppeteer
		WHERE PuppeteerBan <> 'yes'
		ORDER BY PuppeteerLogin");	

	# Title
	print FILE "\n\n--\n-- PUPPETEER\n--\n";

	# Cleanup
	print FILE "DELETE FROM $default.Puppeteer WHERE PuppeteerLogin <> 'system';\n";

	# Loop and display
	while ($res = $statement->fetchrow_hashref())
	{
		# Display
		printf FILE "INSERT INTO $default.Puppeteer (PuppeteerLogin, PuppeteerPassword, PuppeteerName, PuppeteerEMail, PuppeteerVerified, PuppeteerAdminPrivs, PuppeteerSuperPrivs, PuppeteerModPrivs, PuppeteerHome, PuppeteerHomeSpan) VALUES(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s);\n",
			Format($res->{'PuppeteerLogin'}),
			Format($res->{'PuppeteerPassword'}),
			Format($res->{'PuppeteerName'}),
			Format($res->{'PuppeteerEMail'}),
			Format($res->{'PuppeteerVerified'}),
			Format($res->{'PuppeteerAdminPrivs'}),
			Format($res->{'PuppeteerSuperPrivs'}),
			Format($res->{'PuppeteerModPrivs'}),
			Format($res->{'PuppeteerHome'}),
			Format($res->{'PuppeteerHomeSpan'});
	}

	# End
	$statement->finish();


	#
	#
	# Realm Supervisor
	$data->Pull(\$statement, "SELECT
		  RealmName        AS \"RealmName\",
		  PuppeteerLogin   AS \"PuppeteerLogin\",
		  RealmACL         AS \"RealmACL\"
		FROM RealmSupervisor
		ORDER BY RealmName, PuppeteerLogin");

	# Title
	print FILE "\n\n--\n-- REALM SUPERVISOR\n--\n";

	# Cleanup
	print FILE "DELETE FROM $default.RealmSupervisor;\n";

	# Loop and display
	while ($res = $statement->fetchrow_hashref())
	{
		# Display
		printf FILE "INSERT INTO $default.RealmSupervisor (RealmName, PuppeteerLogin, RealmACL) VALUES(%s,%s,%s);\n",
			Format($res->{'RealmName'}),
			Format($res->{'PuppeteerLogin'}),
			Format($res->{'RealmACL'});
	}

	# End
	$statement->finish();


	#
	#
	# Puppet
	$data->Pull(\$statement, "SELECT
		  PuppetName      AS \"PuppetName\",
		  PuppetPool      AS \"PuppetPool\",
		  PuppeteerLogin  AS \"PuppeteerLogin\",
		  PuppetDefault   AS \"PuppetDefault\",
		  PuppetBio       AS \"PuppetBio\",
		  PuppetTag       AS \"PuppetTag\",
		  PuppetTitle     AS \"PuppetTitle\",
		  PuppetGummy     AS \"PuppetGummy\",
		  PuppetPic       AS \"PuppetPic\",
		  PuppetPicHeight AS \"PuppetPicHeight\",
		  PuppetPicWidth  AS \"PuppetPicWidth\",
		  PuppetPicSize   AS \"PuppetPicSize\"
		FROM Puppet
		WHERE PuppetPool NOT IN ('banned')
		ORDER BY PuppeteerLogin, PuppetName");

	# Title
	print FILE "\n\n--\n-- REALM SUPERVISOR\n--\n";

	# Cleanup
	print FILE "DELETE FROM $default.Puppet;\n";

	# Loop and display
	while ($res = $statement->fetchrow_hashref())
	{
		# Display
		printf FILE "INSERT INTO $default.Puppet (PuppetName, PuppetPool, PuppeteerLogin, PuppetShort, PuppetDefault, PuppetBio, PuppetTag, PuppetTitle, PuppetGummy, PuppetPic, PuppetPicHeight, PuppetPicWidth, PuppetPicSize) VALUES(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s);\n",
			  Format($res->{'PuppetName'}),
			  Format($res->{'PuppetPool'}),
			  Format($res->{'PuppeteerLogin'}),
			  Short($res->{'PuppetName'}),
			  Format($res->{'PuppetDefault'}),
			  Format($res->{'PuppetBio'}),
			  Format($res->{'PuppetTag'}),
			  Format($res->{'PuppetTitle'}),
			  Format($res->{'PuppetGummy'}),
			  Format($res->{'PuppetPic'}),
			  Format($res->{'PuppetPicHeight'}),
			  Format($res->{'PuppetPicWidth'}),
			  Format($res->{'PuppetPicSize'});
	}

	# End
	$statement->finish();


	# Close file
	close(FILE);
}
